/*
 * vf.c version 1.1 part of the vfloppy 1.4 package
 *
 * Copyright 2002 by Fred Jan Kraan (fjkraan@xs4all.nl)
 *
 * vf is placed under the GNU General Public License in July 2002.
 *
 *  This file is part of Vfloppy 1.5.
 *
 *  Vfloppy is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Vfloppy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vfloppy; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * vf - used by vfread and vfwrite
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
//#include <asm/errno.h>
#include "vf.h"
#define ENOENT 2

#define PERMS 0664

unsigned char *fileName;
unsigned char dirFileName[12];
unsigned char *imageName;
int fnd;
int imd;
unsigned char blockList[MAXBLOCKCOUNT];
unsigned char data[BLOCKSIZE];
int lastRecCnt;
int entryNo;
int errno;
int fileSize;
int fileBlks;
int fileDirEntries; 
int debug = 3;

/*
 - read arguments
 - check and mount image
 - filename to uppercase and convert to directory format
 - parse image directory and return block list
   abort on file not found.
 - create file and dump the blocks
 - close, unmount, exit
 */

int showDirEntry(unsigned char *dent)
{
	/*
	 * convert data from the directory entry *dent
	 * and represent it as a sortof directory list.
	 * 'local' is for the current directory entry 
	 * only. The global values should reflect the
	 * total size.
	 */
        char filename[13];
	filename[12]=0;
	int localEntryNo, recCnt, blockCnt, localLastRecs, i;
	char attr[3];
	attr[2] = 0;
	int usedBlocks = 0;

        if (dent[DIRUSERNO] != FMTPATT)
	{
                memcpy(&filename[0], &dent[DIRNAMEBASE], 8);
                filename[8] = '.';
                memcpy(&filename[9], &dent[DIREXTBASE], 3);
		filename[9]  &= CHARHIBITMASK;
		filename[10] &= CHARHIBITMASK;
                
		attr[0] = getReadonlyState(dent);
		attr[1] = getSystemFileState(dent);
		
//		recCnt = (((dent[DIRENTRYNOFIELD] & 0Xff) / 2) & EXM) * RECSIZE + (dent[DIRRECS] & 0Xff);
		recCnt = ((dent[DIRENTRYNOFIELD] & 0Xff)* RECSIZE + (dent[DIRRECS] & 0Xff));
		blockCnt = recCnt / RECSPERBLOCK;
		localLastRecs = recCnt % RECSPERBLOCK;
		if (recCnt != 0)
		{
			if (localLastRecs == 0)
			{
				localLastRecs = 16;
			} else {
				blockCnt++;
			}
		}
		if ((dent[DIRUSERNO]  & 0xFF) < HIUSERNO) {
		    printf("%2X:%s %2s ", dent[DIRUSERNO] & 0xFF, filename, attr);
		    printf("%6u bytes ", recCnt * RECSIZE);

		    if (debug > 1)
			printf("(%3u blks, %2u recs in last blk) ", blockCnt, localLastRecs);

		    localEntryNo = getDirEntryInfo(dent);
		    if (dent[DIRENTRYNOFIELD] != 0)
			printf("dir. entry %d", localEntryNo);
		    if (dent[DIRS2] != 0)
			printf("  ** Corrupt Directory Entry **");
		    printf("\n");

		    for (i = DIRBLOCKBASE; i <= (DIRBLOCKBASE + DIRBLOCKSIZE); i++) {
			if (dent[i] != 0)
			    usedBlocks++; /* Used in vfread main */
		    }
		} else {
		    if (debug > 3)
			printf("%2X:%s %2s \n", dent[DIRUSERNO] & 0xFF, filename, attr);
		}
        } 
	return usedBlocks;
}

unsigned char getReadonlyState(unsigned char *dent) 
{
	if ((dent[DIREXTBASE] > CHARHIBITMASK) != 0)
	{
		return('R');
	}
	return(' ');
}

unsigned char getSystemFileState(unsigned char *dent)
{
	if (dent[DIREXTBASE + 1] > CHARHIBITMASK)
	{
		return('S');
	}
	return(' ');
}

int getDirEntryInfo(unsigned char *dent)
{
/*	int localEntryNo = ((dent[DIRENTRYNOFIELD] & 0Xff) + (dent[DIREXTCNTHIGH] & 0Xff) / (EXM + 1));
	if (localEntryNo > 1) localEntryNo--; 
	return(localEntryNo); */
	return (dent[DIRENTRYNOFIELD] / 2 + dent[DIRS2] * 32);
}

int blockRead(int fd, unsigned char *data, unsigned long blockStart, int len)
{
        int err;
        if(lseek(fd, blockStart, 0)==-1)
        {
                perror("vfread: lseek");
                return -1;
        }
        err=read(fd,data,len);
        return err;
}

int blockWrite(int fd, unsigned char *data, unsigned long block, int len)
{
        int err;
        if(lseek(fd, block, 0)==-1)
        {
                perror("vfwrite: lseek");
                return -1;
        }
        err = write(fd,data,len);
        return err;
}

void showDir(int id)
{
	unsigned long dirEntry, offset;
	int entry;
	int err;
	int freeBlocks;
	int usedBlocks = 0;
	
	offset = SECT0HEAD + SECHEADSIZ + (DIRTRK * TRACKSIZE);
	if (debug>1) printf("Reading directory entries from %X\n", (unsigned int)offset);
	for (entry = 0; entry < DIRSECCNT * ENTRYPSEC; entry++) {
		dirEntry = getDirEntryLocation(entry);
		
		err = blockRead(id, data, dirEntry, DIRINCR);
		usedBlocks += showDirEntry(data); 

	}
	freeBlocks = DISKBLOCKCOUNT - usedBlocks;
	printf("%d kByte free (%d blocks, %d records)\n", 
		freeBlocks * 2, 
		freeBlocks,
		freeBlocks * RECSPERBLOCK);

}

int getDirEntryLocation (int count) 
{
	int sector, entry;
	int offset = SECT0HEAD + SECHEADSIZ + (DIRTRK * TRACKSIZE);
	int sectorLocation, entryLocation;
	
	sector = count / ENTRYPSEC;
	sectorLocation = offset + sector * (SECHEADSIZ + SECSIZE);
//	printf("read direcory sector %d at location %X ", sector, sectorLocation);
	entry = count - sector * ENTRYPSEC;
	entryLocation = sectorLocation + entry * ENTRYSIZE;
//	printf(" read directory entry %d at location %X\n", entry, entryLocation);
	return entryLocation;
}

int getBlockLocation(int block)
{
	block--;	// block 0 is directory?
	return BLOCKBASE + block * BRUTOBLKSZ;
}

int mountImage(char *imName)
{
	if((imd = open(imName, O_RDONLY|O_BINARY, 0)) == -1)
	{
		fprintf(stderr," failed to open %s\n",imName);
		exit(1);
	}

	if (debug>1)
		printf("Disk image %s (fd: %d) mounted ok (RO).\n",imName,imd);

	return imd;
}

int mountImageRW(char *imName)
{
	if((imd = open(imName,O_WRONLY|O_RDONLY|O_BINARY,0)) == -1)
	{
		fprintf(stderr," failed to open %s\n",imName);
		exit(1);
	}
	if (debug>1)
		printf("Disk image %s (fd: %d) mounted ok (RW).\n",imName,imd);

	return imd;
}

void convertFileName(unsigned char *dFName, unsigned char *flName) 
{
	/* conversie fileName dirFileName
	   - to uppercase
	   - add spaces to basename to 8 chars
	   - remove dot
	   - add spaces to extension to 3 chars
	 */
	int i = 0;
	int j = 0;
	int c;
	for (c = 0; c < 11; c++)
		dFName[c] = ' ';
	dFName[11] = '\0';

	/* walk though the array to fill and check the
	 * input array:
	 * - end of string: 	exit
	 * - a dot:		write the rest at the extension location
	 * Most problems are not checked here, but will hopefully fail
	 * nicely at filename comparison.
	 */
	for (i = 0; i < 12; i++)
	{
		if (flName[i] == '\0')
			break;
		if (flName[i] == '.')
		{
			j = 8;
		} else {
			if (j < 11)
			{
				dFName[j] = toupper(flName[i]);
				j++;
			}	
		}
	} 

	if (debug>2)
		printf("Converted filename: \'%s\'\n", dFName); 

}

int cmpEntry(unsigned char *dent, unsigned char *dFlName)
{
	char cmpFileName[12];

	/*
	 * - get entry
	 * - if match: return 1 else 0
	 */

	if (dent[DIRUSERNO] == FMTPATT)
	{

//		if (debug>2)
//			printf("empty or deleted file entry\n");

		return 0;
	}
	memcpy(&cmpFileName[0], &dent[DIRNAMEBASE], DIRNAMESIZE);
	cmpFileName[DIRNAMESIZE] = '\0';

	cmpFileName[8] = 0x7f & cmpFileName[8];	/* clear R/O bit */
	cmpFileName[9] = 0x7f & cmpFileName[9];	/* clear SYS bit */
	if (strcmp(cmpFileName, (const char*)dFlName) == 0)
	{

	if (debug>1)
		printf("Found file \'%s\' entryNo %d\n", dFlName, dent[DIRENTRYNOFIELD]);

        	return 1;
	} else {
		return 0;
	}
}

void updateBlockList(unsigned char *dent, s_copySet *cs)
{
	int localEntryNo = 0;
	int recCnt, localLastRecs;
	
	int i = 0;
	int blockListIndex = 0;
	
	while (cs->blockList[blockListIndex] != 0) blockListIndex++;
	
	if (debug>1) printf(" found empty location in blockList at: %d\n", blockListIndex);

	/* selects the part of the BlockList the blocks are copied to */

	for  (i = 0; i < DIRBLOCKSIZE; i++)
	{
		if (dent[i + DIRBLOCKBASE] == '\0') break;
//		blockList[i] = dent[DIRBLOCKBASE + i];
		cs->blockList[i + blockListIndex] = dent[DIRBLOCKBASE + i];
		if (debug>1) printf("%d: %X; ", i + blockListIndex , dent[i + DIRBLOCKBASE]);
	}

	printf("\n"); 

	/* 
	 * determine the number of used records in the
	 * last allocated block. Done for all entries,
	 * but the globals contain the right data in the end.
	 */
	recCnt = (dent[DIRENTRYNOFIELD] & EXM) * 128 + dent[DIRRECS];
               localLastRecs = recCnt % RECSPERBLOCK;
               if (localLastRecs == 0)
               {
                       localLastRecs = DIRBLOCKSIZE;
               }

	if (localEntryNo > entryNo)
	{
		entryNo = localEntryNo;
		lastRecCnt = dent[DIRRECS];
		if (debug>2)
			printf("updated entryNo: %d and lastRecCnt: %d\n", entryNo, lastRecCnt);
	}
}

int findFile(int id, unsigned char *dFName, s_copySet *cs)
{
	unsigned long dirEntry;
	int entryCnt = 0;
	int err;

	/* - loop 64 entries
	 * - start at 0x00080, increment with 0x00020
	 */
	int i;
	for (i = 0; i < DIRSECCNT * ENTRYPSEC; i++)  // iterate through directory entries
	{
//		if (debug>2)
//			printf("comparing %s with dir.entry %2d@%lX\n", dFName, i, dirEntry); 

//		dirEntry = (DIRBASE + i * DIRINCR);
		dirEntry = getDirEntryLocation(i);
		err = blockRead(id, data, dirEntry, DIRENTRYCOUNT);
		if (cmpEntry(data, dFName) == 1)
		{
			updateBlockList(data, cs);
        		entryCnt++;
		}

 	}

	if (debug>1)
		printf("Found entries: %d\n", entryCnt);

	return entryCnt;
}

int createFile(unsigned char *fileName, s_copySet *cs)
{
	int i, fd;
	unsigned int blkAddr; 
	unsigned int recCnt;
	/*
	 * for each block until block number is 0:
	 * - translate block number to address in image
	 * - get block and write it to the file
	 */

	if (unlink((const char *)fileName) != 0)
	{
	    if (debug>1)
		printf("attempt to unlink current file: %s....\n", fileName);

		if (errno != ENOENT && errno != 0)
		{
			printf("problem deleting %s in file system. Error %d. Exiting\n", fileName, errno);
			exit(errno);
		} else {
			if (debug>1)
				printf(" file didn't exist in file system\n");
		}

	    }
	    fd = open((const char *)fileName, O_WRONLY|O_BINARY, PERMS);
	    if (fd < 0) 
	    {
		printf("Error opening file %s\n", fileName);
	return 1;
	} 
	if (debug>1)
	{
		printf("writing file: %s\n", fileName);
		printf("Blocks: ");
	}
	for (i = 0; i < MAXBLOCKCOUNT; i++)
	{
		if (cs->blockList[i] == '\0')
		{
			break;
		}
		blkAddr = getBlockLocation(cs->blockList[i]);

		if (debug>1)
			printf("%X: (0x%5X) ", cs->blockList[i], blkAddr);
	
		if (cs->blockList[i + 1] == '\0')
		{
			recCnt = (lastRecCnt & 0x0F) ? (lastRecCnt & 0x0F) : 0x10;
			if (debug>1) printf(" Last block: 0x%X records\n", recCnt);
			cpPartialBlock(blkAddr, recCnt , fd);
		} else {
			cpFullBlock(blkAddr, fd);
		}
	}
	if (debug>1)
	    printf("\n");

	close(fd); 
	return 0;
}

void cpFullBlock(int blkAddress, int fd) 
{
	int sectorAddress, sector;
	for (sector = 0; sector < SECPERBLK; sector++)
	{
		sectorAddress = blkAddress + SECHEADSIZ + (BRUTOSECSIZE * sector);
		blockRead(imd, data, sectorAddress, SECSIZE);
		if (debug>2) printf(" Copying sectors: 0x%X bytes from 0x%X\n", SECSIZE, sectorAddress);
		write(fd, data, SECSIZE);
	}
}

void cpPartialBlock(int blkAddress, int recs, int fd)
{
	int recordAddress, record;
	for (record = 0; record < recs; record++)
	{
		recordAddress = blkAddress + SECHEADSIZ + (BRUTOSECSIZE * (record / 2) + record%2 * RECSIZE);
		blockRead(imd, data, recordAddress, RECSIZE);
		if (debug>2) printf(" Copying records: 0x%X bytes from 0x%X\n", RECSIZE, recordAddress);
		write(fd, data, RECSIZE);
	}
}

int checkFileNSize(unsigned char *fn)
{
	struct stat fileInfo;
	int bSize, size = 0; 
	int statRet;
	
	statRet = stat((const char*)fn, &fileInfo);

/*	printf("statRet returned %d \n", statRet); */
	if (statRet != 0)
	{
		if (statRet == -1)
		{
			fprintf(stderr,"file %s not found. Exiting...\n", fn);
		} else {
			fprintf(stderr,"error statting %s. Exiting...\n", fn);
		}
		exit(statRet);
	}
	size = fileInfo.st_size;
	if (size == 0)
	{
		printf("file is empty. Exiting...\n");
		exit(0);
	}
	bSize = (size / BLOCKSIZE) + 1;
	if (size % BLOCKSIZE == 0)
		bSize--;
	if (debug>1)
		printf("file is %d bytes, %d blocks on floppy image\n", size, bSize);
	
	return size;
}

int countUsedImgDirEntries(int id, unsigned char *dFn)
{
	int i, err;
	unsigned long dirEntry;
	int count = 0;

        for (i = 0; i < DIRENTRYCOUNT; i++)
        {
		dirEntry = getDirEntryLocation(i);
//	  dirEntry = (DIRBASE + i * DIRINCR);
                err = blockRead(id, data, dirEntry, DIRENTRYSIZE);
                if (data[DIRUSERNO] != FMTPATT) count++;
	}
	if (debug>1)
		printf("Number of dir entries used: %d\n", count);
	return count;
}

int countUsedImgBlocks(int id, unsigned char *dFn)
{
        int i, j, err, blkLoop;
	unsigned long dirEntry;
	int size = 0;
	for (i = 0; i < DIRENTRYCOUNT; i++)
	{
		dirEntry = getDirEntryLocation(i);
		err = blockRead(id, data, dirEntry, DIRENTRYSIZE);
		if (data[DIRUSERNO] != FMTPATT)		// extend is in use
		{
			for (j = 0; j < DIRBLOCKSIZE; j++)
			{
				blkLoop = DIRBLOCKBASE + j;
				if (data[blkLoop] != 0)
				{
					size++;
				}
				else break;	// skip rest of entry

				if (debug>2)
					printf("+%2X=%X:%d ",blkLoop, data[blkLoop], size);

			}
			if (debug>2)
				printf("\n");

		}
	}
	if (debug>2)
		printf("\n size: %d\n", size);

	return size;
}

int countBlocks(unsigned char *blkList)
{
        int i = 0;
	int blockCnt = 0;

        while (blkList[i] != 0)
        {
		i++;
		blockCnt++;
	}
	if (debug>1)
		printf("  %d blocks counted for file in image\n", blockCnt);
	return blockCnt;
}

void eraFile(int id, unsigned char *dFName)
{
        unsigned long dirEntry;
        int i = 0;
        int err;
        unsigned char FmtChar[1];
        FmtChar[0] = FMTPATT;

        /* - loop 64 entries
         * - start at 0x00080, increment with 0x00020
         */

        for (i = 0; i < DIRENTRYCOUNT; i++)
        {
		dirEntry = getDirEntryLocation(i);
                err = blockRead(id, data, dirEntry, DIRENTRYCOUNT);
		dFName[8] = 0x7f & dFName[8];	/* clear R/O bit */
		dFName[9] = 0x7f & dFName[9];	/* clear SYS bit */
                if (cmpEntry(data, dFName) == 1)
                {

			if (debug>2)
                	        printf("erasing entry %d of \'%s\'\n", i, dFName);

			blockWrite(id, FmtChar, dirEntry, 1); 
                }
        }
}

void getFreeBlockList(int id, s_copySet * cs)
{
	/* - make a list of all blocks
	 * - parse all used directory entries, zeroing all used blocks
	 * - summarize all remaining bloks in blockList[]
	 */

	int i, j, err;
	unsigned char tmpBlk;
	char allBlockList[DISKBLOCKCOUNT];
	unsigned long dirEntry;

	memset(&cs->blockList[0], 0x0, MAXBLOCKCOUNT);

	allBlockList[0] = 0;  /* block zero always used by directory */
	for (i = 1; i < DISKBLOCKCOUNT; i++)
	{
		allBlockList[i] = i & 0xFF;
	}
	for (i = 0; i < DIRENTRYCOUNT; i++)
        {
//                dirEntry = (DIRBASE + i * DIRINCR);
                dirEntry = getDirEntryLocation(i);
                err = blockRead(id, data, dirEntry, DIRENTRYCOUNT);
		if (data[DIRUSERNO] != FMTPATT)
		{
			for (j = 0; j < DIRBLOCKSIZE; j++)
			{
				tmpBlk = data[DIRBLOCKBASE + j];
				if (tmpBlk != 0)
					allBlockList[tmpBlk] = 0;
			}
		}
	}
	if (debug>2) printf("Free blocks: ");
	j = 0;
	for (i = 0; i < DISKBLOCKCOUNT; i++)
	{

		if (debug>2)
			printf("%2X:", j);
		if (allBlockList[i] != 0)
		{
			cs->blockList[j] = allBlockList[i];
			if (debug>2)
				printf("%2X ", allBlockList[i] & 0xFF);
			j++;
		} else if (debug>2) printf("- ");
	}
	if (debug>2)
		printf("\n");
}

int writeDirEntry(int id, unsigned char *dirEnt)
{
	/* 
	 * - loop through dir entries
	 * - when empty:
	 *   - write new entry
	 */
        unsigned long dirEntry;
        int i;
        int err;

        for (i = 0; i < DIRENTRYCOUNT; i++)
        {
                dirEntry = getDirEntryLocation(i);
                err = blockRead(id, data, dirEntry, DIRENTRYSIZE);
		if (data[DIRUSERNO] == FMTPATT)
		{
			blockWrite(id, &dirEnt[0], dirEntry, DIRENTRYSIZE);
			return i;
		}
	} 
	return 0;
}

void writeDir(int id, unsigned char *dFn, unsigned char *fn, s_copySet * cs)
{
	/*
	 * - loop through dir entry sized parts (16 * BLOCKSIZE)
	 *   - fill in file details in temp array
	 *   - write array to image
	 */
	int dirEntryCnt;
	int i;
	int lastEntry = 1;
	int fileRecCnt;
	int dirEntNo;
	unsigned char tmpDirEntry[32];

	// How many directory extends do we need?
	dirEntryCnt = (fileSize / (DIRBLOCKSIZE * BLOCKSIZE)) + 1;
	if (fileSize % (DIRBLOCKSIZE * BLOCKSIZE) == 0)
		dirEntryCnt--;
	fileRecCnt = (fileSize / RECSIZE) + 1;
	if (fileSize % RECSIZE == 0)
		fileRecCnt--;
 	if (debug>2)
		printf("dirEntryCount: %d; fileRecCount: %d\n", dirEntryCnt, fileRecCnt);
	

	for (i = 0; i < dirEntryCnt; i++)
	{
		if (i + 1 == dirEntryCnt)
			lastEntry = 0;
		memset(&tmpDirEntry[DIRUSERNO], 0x0, DIRENTRYSIZE);
		tmpDirEntry[0] = 0;
		memcpy(&tmpDirEntry[DIRNAMEBASE], &dirFileName[0], DIRNAMESIZE);
		tmpDirEntry[DIRENTRYNOFIELD] = (i * 2) + lastEntry;
		if (lastEntry == 0)
		{
/*			tmpDirEntry[DIRRECS] = fileRecCnt % RECSIZE; */
			tmpDirEntry[DIRRECS] = fileRecCnt;
		} else {
			tmpDirEntry[DIRRECS] = MAXRECCOUNT;
		}
		memcpy(&tmpDirEntry[DIRBLOCKBASE], &cs->blockList[i * DIRBLOCKSIZE], DIRBLOCKSIZE);
		dirEntNo = writeDirEntry(id, &tmpDirEntry[0]);
 		if (debug>2)
			printf("wrote to dir.entry: %d\n", dirEntNo);
	}
}

int writeBlocks(unsigned char *fileName, s_copySet * cs)
{
	int fd, i, currentBlockSize, err;
	unsigned long blkAddr;

        fd = open((const char *)fileName, O_RDONLY|O_BINARY);
	int fileInBlocks = fileSize / BLOCKSIZE;
	if (fileSize % BLOCKSIZE != 0) fileInBlocks++;
        if (fd < 0)
        {
                printf("Error opening file %s\n", fileName);
                return 1;
        }
	if (debug>1)
	{
        	printf("reading file: %s\n", fileName);
      	  	printf("Blocks: %d / %d = %d\n", fileSize, BLOCKSIZE, fileInBlocks);
	}

        for (i = 0; i < fileInBlocks; i++)	// skip 0, it is the directory
        {
                if (cs->blockList[i] == '\0')
                        break;
		blkAddr = getBlockLocation(cs->blockList[i]);
//                blkAddr = BLOCKBASE + blockList[i] * BLOCKSIZE;
		if (debug>2)
	                printf("%X: %4lX ", cs->blockList[i], blkAddr);

//                if (cs->blockList[i + 1] == '\0')
		  if (i + 1 == fileInBlocks)
		  { // last block 
                        currentBlockSize = fileSize % BLOCKSIZE;
			if (debug>2)
                        	printf("Last block size: %d\n", currentBlockSize);
			memset(&data[0], 0x0, BLOCKSIZE);
			
			int recs = (currentBlockSize + RECSIZE / 2) / RECSIZE ;
//			int recs = currentBlockSize % RECSIZE ;
			
			if (debug>2)
                        	printf("Record count: %d\n", recs);
			int recordAddress, record;
			for (record = 0; record < recs; record++)
			{
				memset(&data[0], 0xE5, RECSIZE * 2);
				err = read(fd, data, RECSIZE);
				if (err == 0) break;

				recordAddress = blkAddr + SECHEADSIZ + (BRUTOSECSIZE * (record / 2) + record%2 * RECSIZE);
				blockWrite(imd, data, recordAddress, RECSIZE);
				if (debug>2) 
				  printf(" Copying (bytes:%d) records: 0x%X bytes to 0x%X\n", err, RECSIZE, recordAddress);
			}
			
			
                } else { // other blocks
                        currentBlockSize = BLOCKSIZE;
			int sectorAddress, sector;
			for (sector = 0; sector < SECPERBLK; sector++)
			{
				err = read(fd, data, SECSIZE);
				sectorAddress = blkAddr + SECHEADSIZ + (BRUTOSECSIZE * sector);
				blockWrite(imd, data, sectorAddress, SECSIZE);
				if (debug>2) printf(" Copying sectors: 0x%X bytes to 0x%X\n", SECSIZE, sectorAddress);
			}

                }
		



		
/*		err = read(fd, data, currentBlockSize);
		if (err == -1)
		{
			printf("error while reading %s. Exiting...\n", fileName);
			exit(1);
		}
                blockWrite(imd, data, blkAddr, currentBlockSize); */
        }
	return 0;
}

