/*
 *	Vfloppy image structure
 */

/* d88 struct */
#define SECT0HEAD   0x02B0
#define SECHEADSIZ  0x10
#define SECSIZE     0x100
#define BRUTOSECSIZE 0x110

#define DIRTRK      0x04
#define DIRSECCNT   0x08
#define ENTRYPSEC   0x08
#define ENTRYSIZE   0x20
#define SECPERTRK   0x10
#define HEADS       0x02
#define TRACKSIZE   (SECHEADSIZ + SECSIZE) * SECPERTRK * HEADS

#define BLOCKBASE   0x9330
#define BRUTOBLKSZ  (SECHEADSIZ + SECSIZE) * 8 // 8 sectors per block
#define SECPERBLK   0x08
#define SECTORCOUNT 0x10
#define TRACKCOUNT  0x28

/*
 * UU F1 F2 F3 F4 F5 F6 F7 F8 T1 T2 T3 EX S1 S2 RC   .FILENAMETYP....
 * AL AL AL AL AL AL AL AL AL AL AL AL AL AL AL AL   ................
 */

/* generic directory information, offset from start of image */
#define DIRBASE		0x80	/* 128 byte - starting point dir. in image         */
#define DIRINCR		0x20	/* 32 byte - offset per directory entry	           */
#define DIRENTRYSIZE	0x20	/* 32 byte - size of a directory entry	           */
#define DIRENTRYCOUNT	0x40	/* 64 - number of directory entries	           */

//#define BLOCKBASE	0x0080	/* starting point zeroth block. Note block 0 is    */
                                /* reserved for the directory */
#define BLOCKSIZE	0x0800	/* 2 kByte 			                   */
#define RECSIZE		0x80	/* 128 byte		     	                   */
#define RECSPERBLOCK	0x10	/* 16 - number of records per allocation block     */
#define MAXRECCOUNT	0x80	/* 128 records per directory entry	           */
#define MAXBLOCKCOUNT	256	/* theoretical maximal number of allocation blocks */

#define DISKBLOCKCOUNT	0x8b 	/* 139 all.blocks, 278 kByte - actual disk size    */
 
/* directory entry information, offset from start of dir. entry                    */
#define DIRUSERNO	0x00   	/* UU byte		                           */
#define DIRNAMEBASE	0x01   	/* Fx bytes		                           */
#define DIREXTBASE	0x09	/* Tx bytes		                           */
#define DIRNAMESIZE	11  	/* Fx + Tx byte count	                           */
#define DIRENTRYNOFIELD	0x0C  	/* EX byte bit 0: 0=one entry, 1, 2, ... entry number   * 
				 *         bits 1-7 used for count                 */
#define DIREXTCNTHIGH	13	/* S1 byte extra directory entry count bits        */
#define DIRS2		14	/* S2 byte		                           */
#define DIRRECS		15	/* RC byte		                           */
#define SYSBITROLOC	0x09	/* T1, bit 7. Read Only bit                        */
#define SYSBITSYSLOC	0x0A	/* T2, bit 7. System file bit                      */
/* AL bytes                                                                        */
#define DIRBLOCKBASE	16  	/* first AL byte	                           */
#define DIRBLOCKSIZE	16  	/* AL byte count	                           */

#define EXM		0x1f	/* Extend mask		                           */
#define FMTPATT		0xe5	/* disk formatting pattern
				 * also used to delete directory entry             */
#define DEBUG		0
#define CHARHIBITMASK   0x7F
#define HIUSERNO        16

// track header
struct d88sct_t {
	unsigned char c, h, r, n;
	unsigned short nsec;
	unsigned char dens, del, stat;
	unsigned char rsrv[5];
	unsigned short size;
}; 

typedef struct 
{
	unsigned char blockList[MAXBLOCKCOUNT];
	unsigned int recsInLastBlk;
} s_copySet ;



//extern unsigned char *fileName;
//extern unsigned char dirFileName[];
//extern unsigned char *imageName;
extern int  fnd;
extern int  imd;
//extern unsigned char blockList[];
extern unsigned char data[];
extern int  lastRecCnt;
extern int  entryNo;
extern int  totalUsedBlockCnt;
extern int  errno;
extern int  fileSize;
extern int  fileBlks;
extern int  fileDirEntries;
extern int  debug;

extern int showDirEntry(unsigned char *dent);
extern unsigned char getReadonlyState(unsigned char *dent);
extern unsigned char getSystemFileState(unsigned char *dent);
extern int  getDirEntryInfo(unsigned char *dent);
extern int  blockRead(int fd, unsigned char *data, unsigned long blockStart, int len);
extern int  blockWrite(int fd, unsigned char *data, unsigned long block, int len);
extern void showDir(int id);
extern int  mountImage(char *imName);
extern int  mountImageRW(char *imName);
extern void convertFileName(unsigned char *dFName, unsigned char *flName);
extern int  cmpEntry(unsigned char *dent, unsigned char *dFlName);
extern void updateBlockList(unsigned char *dent, s_copySet *cs);
extern int  findFile(int, unsigned char*, s_copySet * cs);
extern int  createFile(unsigned char *fileName, s_copySet * cs);
extern int  checkFileNSize(unsigned char *fn);
extern int  countUsedImgDirEntries(int id, unsigned char *dFn);
extern int  countUsedImgBlocks(int id, unsigned char *dFn);
extern int  countBlocks(unsigned char *blkList);
extern void eraFile(int id, unsigned char *dFName);
extern void getFreeBlockList(int id, s_copySet * cs);
extern int  writeDirEntry(int id, unsigned char *dirEnt);
extern void writeDir(int id, unsigned char *dFn, unsigned char *fn, s_copySet * cs);
extern int  writeBlocks(unsigned char *fileName, s_copySet * cs);
extern int getDirEntryLocation (int count); 
extern void cpFullBlock(int blkAddress, int fd);
extern void cpPartialBlock(int blkAddress, int recs, int fd);
