/*
 * vformat.c version 1.0 part of the vfloppy 1.3 package
 *
 * Copyright 1996 Justin Mitchell (madmitch@discordia.org.uk) and friends.
 *
 * This version delivered in 2002 by Fred Jan Kraan (fjkraan@xs4all.nl)
 * Incorporated vformat into the vfman suite.
 *
 * vfread is placed under the GNU General Public License in July 2002.
 *
 *  This file is part of Vfloppy 1.3.
 *
 *  Vfloppy is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Vfloppy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vfloppy; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * vformat - create a file of image size filled with 0xe5's
 *           The size is BLOCKSIZE * DISKBLOCKCOUNT + RECSIZE
 *           This is because the first four tracks are reduced
 *           to one sector as they are used for CP/M boot code,
 *           and the PX line do not boot from disk.
 * 
 * vformat <imageName>
 */

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
//#include <asm/errno.h>
#include <stdio.h>
#include <fcntl.h>
#include "vf.h"

#define PERMS 0664

int main(int argc,char **argv)
{
	unsigned char sector, head, track;
	FILE *image;

	char *headerName = "d88header.bin";
	int fileSize;
	
	// sector header
	struct d88sct_t sectorHeader;
	sectorHeader.c      = 0;
	sectorHeader.h      = 0;
	sectorHeader.r      = 0;
	sectorHeader.n      = 1;
	
	sectorHeader.nsec    = 0x1000;
	sectorHeader.dens    = 0;
	sectorHeader.del     = 0;
	
	sectorHeader.stat    = 0;
	sectorHeader.rsrv[0] = 0;
	sectorHeader.rsrv[1] = 0;
	sectorHeader.rsrv[2] = 0;
	
	sectorHeader.rsrv[3] = 0;
	sectorHeader.rsrv[4] = 0;
	sectorHeader.size    = 0x0100;


	unsigned char buff[SECSIZE];
	unsigned char dskhead[SECT0HEAD];


	if (argc<2) 
	{
		fprintf(stderr,"Usage: %s <filename>\n",argv[0]);
		exit(0);
	}
	
	
	FILE *imd = fopen(headerName,"rb");
	if (imd == NULL)
	{
		fprintf(stderr," failed to open %s\n", headerName);
		exit(1);
	}
	fileSize = checkFileNSize((unsigned char *)headerName);
	fread(&dskhead, fileSize, 1, imd);
	

	if (debug>1)
		printf("Image header %s loaded ok (RO).\n",headerName);
	
	memset(buff, 0xe5, SECSIZE);
	
	image = fopen(argv[1], "wb");
	if (image == NULL) exit(2);
	printf("File %s opened fir writing\n", argv[1]);
	
	fwrite(&dskhead, SECT0HEAD, 1, image);
	for (track = 0; track < TRACKCOUNT; track++)
	{
		sectorHeader.c = track;
		for (head = 0; head < HEADS; head++)
		{
			sectorHeader.h = head;
			for (sector = 1; sector <= SECTORCOUNT; sector++)
			{
				sectorHeader.r = sector;
				fwrite(&sectorHeader, SECHEADSIZ, 1, image);
				fwrite(&buff, SECSIZE, 1, image);
			}
		}
		printf("Track: %d; Head: %d; Sector: %d \n", track, head, sector);
	}
	
	fclose(image);
	return 0;
}
