/*
 * vfread.c version 1.2 part of the vfloppy 1.4 package
 *
 * Copyright 2002 by Fred Jan Kraan (fjkraan@xs4all.nl)
 *
 * vfread is placed under the GNU General Public License in July 2002.
 *
 *  This file is part of Vfloppy 1.5.
 *
 *  Vfloppy is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Vfloppy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vfloppy; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * vfread - reads directory and files from disk images
 *          used by the vfloppy package.
 * usage: vfread <imageName> [<fileName>]
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "vf.h"

#define PERMS 0664

/*
 - read arguments
 - check and mount image
 - filename to uppercase and convert to directory format
 - parse image directory and return block list
   abort on file not found.
 - create file and dump the blocks
 - close, unmount, exit
 */


int main(int argc, char *argv[])
{
	extern char *imageName;
	extern unsigned char *fileName;
	extern unsigned char dirFileName[];

	s_copySet copySet;
	
	int c;
	for (c = 0; c < MAXBLOCKCOUNT; c++) copySet.blockList[c] = 0;
	copySet.recsInLastBlk = 0;
	
	s_copySet *cs = &copySet;	// pointer alias, handy for passing

	entryNo = -1;
	lastRecCnt = -1;
	memset(&data[0], 0x0, BLOCKSIZE);
	if(argc<2 || argc> 3)
	{
		fprintf(stderr, "%s <imageName> [<fileName>]\n", argv[0]);
		fprintf(stderr, "d88read 0.1 (c)2010 F.J. Kraan\n");
		exit(1);
	}
	imageName = argv[1];
	imd = mountImage(imageName);

        if(argc == 3) {
	    /* Read file mode */
		fileName = (unsigned char*)argv[2];
	} else {
	    /* Directory read mode */
		if (debug>1)
			printf("To directory mode\n");

		showDir(imd);
		exit(0);
	}
	convertFileName(dirFileName, fileName);

//	if (findFile(imd, dirFileName) > 0)
//		createFile(fileName); 
	 if (findFile(imd, dirFileName, cs) > 0) {
		createFile(fileName, cs); 
		c = 0;
		if (debug>2) {
			while (cs->blockList[c] != 0) {printf("%X, ", cs->blockList[c]); c++; }
			printf("count = %d\n", c);
		}
	 }

	exit(0);
}
